/*****************************************************************************
**  Copyright (C) 2004                                                      **
**  University of Tennessee                                                 **
**                                                                          **
**  See the file COPYRIGHT in the package base directory for details        **
*****************************************************************************/

#include "cone_cnode.h"
#include "cone_error.h"
#include "cone_cndata.h"
#include <string.h>
#include <stdlib.h>

struct cone_cnode
{
  long    id      ;
  long    csite_id;
  long    parent_id; 
  long*   child_idv;
  long    nchilds;
  long    tag;
  struct cone_cndata*  data;
};

struct cone_cnode*   
cone_cnode_create(char* cnode_spec)
{
  int i ;
  struct cone_cnode* cnode = malloc(sizeof(struct cone_cnode));
 
  if ( cnode == NULL )
   cone_error();
  
  if ( strcmp(strtok(cnode_spec, " "), "CNODE") != 0 )
    cone_error_msg("invalid call-graph node  specification string");
  
  cnode->id        = atol(strtok(NULL, " "));
  cnode->csite_id  = atol(strtok(NULL, " "));
  cnode->parent_id = atol(strtok(NULL, " "));
  cnode->nchilds   = atol(strtok(NULL, " ")); 
 
  if (cnode->nchilds > 0)
    {
      cnode->child_idv = (long*) calloc(cnode->nchilds, sizeof(long));
      if (cnode->child_idv == NULL)
	cone_error();
    }
  else
    {
      cnode->child_idv = NULL;
    }  

  for ( i =0; i < cnode->nchilds; i++)
    cnode->child_idv[i] = atol(strtok(NULL, " "));
 
  
  cnode->tag = -1;

  cnode->data = cone_cndata_create(cnode->id);
 
  return cnode;
}

void              
cone_cnode_delete(struct cone_cnode* cnode)
{
  free(cnode->child_idv); 
  free(cnode->data);
  free(cnode);
}

long
cone_cnode_id(struct cone_cnode* cnode)
{
  return cnode->id ; 
}

long              
cone_cnode_csite_id(struct cone_cnode* cnode)
{
  return cnode->csite_id;
}

long              
cone_cnode_num_childs(struct cone_cnode* cnode)
{
  return cnode->nchilds;
}

long              
cone_cnode_parent_id(struct cone_cnode* cnode)
{
  return cnode->parent_id;
}

long  
cone_cnode_child_id(struct cone_cnode* cnode, long i)
{
  return cnode->child_idv[i];
}

long
cone_cnode_get_tag(struct cone_cnode* cnode)
{
  return cnode->tag;
}

void              
cone_cnode_set_tag(struct cone_cnode* cnode, long tag)
{
  cnode->tag = tag;
}


void
cone_cnode_set_data(struct cone_cnode* cnode,struct cone_cndata* temp)
{
  cnode->data = temp ;
}

struct cone_cndata*
cone_cnode_get_data(struct cone_cnode* cnode )
{
  return cnode->data  ;
}


void             
cone_cnode_fprintf(FILE* stream, struct cone_cnode* cnode)
{
  int i;
  
  fprintf(stream, 
	  "CNODE %d %d %d %d",
          cnode->id, 
	  cnode->csite_id,
	  cnode->parent_id,
          cnode->nchilds);  
  
  for ( i =0; i < cnode->nchilds; i++ )
    fprintf(stream, " %d", cnode->child_idv[i]); 
  fprintf(stream, "\n");
}



