/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee                                                **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include "cone_csite.h"
#include "cone_error.h"

#include <string.h>
#include <stdlib.h>

struct cone_csite
{
  long id;
  long index;
  long mod_id;
  long line;
  long caller_id;
  long callee_id;
  long ncnodes;
  long* cnode_idv; 
  long tag;
};

struct cone_csite*   
cone_csite_create(char* csite_spec)
{
  int i ;
  struct cone_csite* csite = malloc(sizeof(struct cone_csite));
 
 
  if ( csite == NULL )
    cone_error();

  if ( strcmp(strtok(csite_spec, " "), "CALLSITE") != 0 )
    cone_error_msg("invalid call-site specification string");

  
  csite->id        = atol(strtok(NULL, " "));
  csite->index     = atol(strtok(NULL, " "));
  csite->mod_id    = atol(strtok(NULL, " "));
  csite->line      = atol(strtok(NULL, " "));
  csite->caller_id = atol(strtok(NULL, " "));
  csite->callee_id = atol(strtok(NULL, " "));
  csite->ncnodes   = atol(strtok(NULL, " ")); 
  
  if ( csite->ncnodes > 0 )
    {
      csite->cnode_idv = (long*) calloc(csite->ncnodes, sizeof(long));
      if ( csite->cnode_idv == NULL )
	cone_error();
    }
  else
    {
      csite->cnode_idv = NULL;
    }
  
  for ( i = 0; i < csite->ncnodes; i++)
    csite->cnode_idv[i] = atol(strtok(NULL, " "));


  csite->tag = -1;

  return csite;
}

void              
cone_csite_delete(struct cone_csite* csite)
{
  free(csite->cnode_idv);
  free(csite);
}

long              
cone_csite_id(struct cone_csite* csite)
{
  return csite->id;
}

long              
cone_csite_index(struct cone_csite* csite)
{
  return csite->index; 
}

long
cone_csite_mod_id(struct cone_csite* csite)
{
  return csite->mod_id;
}

long              
cone_csite_line(struct cone_csite* csite)
{
  return csite->line;
}

long              
cone_csite_caller_id(struct cone_csite* csite)
{
  return csite->caller_id;
}

long              
cone_csite_callee_id(struct cone_csite* csite)
{
  return csite->callee_id;
}


long              
cone_csite_num_cnodes(struct cone_csite* csite)
{
  return csite->ncnodes;
}

long  
cone_csite_cnode_id(struct cone_csite* csite, long i)
{
  return csite->cnode_idv[i];
}

long              
cone_csite_get_tag(struct cone_csite* csite)
{
  return csite->tag;
}

void              
cone_csite_set_tag(struct cone_csite* csite, long tag)
{
  csite->tag = tag;
}

void             
cone_csite_fprintf(FILE* stream, struct cone_csite* csite)
{
  int i;
  
  fprintf(stream, 
	  "CALLSITE %d %d %d %d %d %d %d", 
	  csite->id, 
          csite->index,
	  csite->mod_id, 
	  csite->line, 
	  csite->caller_id, 
	  csite->callee_id,
          csite->ncnodes); 
  for ( i = 0; i < csite->ncnodes; i++ )
    fprintf(stream, " %d", csite->cnode_idv[i]);
  fprintf(stream,"\n"); 
 /*  fprintf(stream, " tag = %d\n", csite->tag); */
}
