/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee                                                **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include "cone_metric.h"

#include "papi.h"
#include <stdlib.h>
#include <string.h>

#include "cone_error.h"

#define MAX_LINE_LENGTH 8192

struct cone_metric
{
  int id;
  char* name;
  char  descr[PAPI_MAX_STR_LEN];
  int   papi_code;
};

struct cone_metric* cone_metric_create(int met_id, const char* name, int papi_code)
{
  struct cone_metric* met; 
  int retval;
  char tmp[PAPI_MAX_STR_LEN];
  char errstring[PAPI_MAX_STR_LEN];

    
  met = (struct cone_metric*) malloc(sizeof(struct cone_metric));
  if ( met == NULL )
    cone_error();

  met->id = met_id;
  
  met->name = (char*) calloc(strlen(name) + 1, sizeof(char));
  if ( met->name == NULL )
    cone_error();      
  memcpy(met->name, name, strlen(name) + 1);

  met->papi_code = papi_code;
  
  tmp[0] = 0; /* reset tmp to an emty string */

 /*  retval = PAPI_describe_event(tmp, &papi_code, met->descr); */
/*   if ( retval != PAPI_OK ) */
/*     { */
/*       PAPI_perror(retval, errstring, PAPI_MAX_STR_LEN); */
/*       cone_error_msg("%s", errstring); */
/*     } */
 
  return met;
}

void cone_metric_free(struct cone_metric* met)
{
  free (met->name);
  free(met);
}

const char* cone_metric_name(struct cone_metric* met)
{
  return met->name;
}

const char* cone_metric_descr(struct cone_metric* met)
{
  return met->descr;
}

int cone_metric_papi_code(struct cone_metric* met)
{
  return met->papi_code;
}

void cone_metric_fprintf(FILE* stream, struct cone_metric* met)
{
  fprintf(stream, 
	  "METRIC %d %s occ excl %s\n", 
	  met->id, 
	  met->name,
       	  met->descr);  
}


